# -*- coding: utf-8 -*-
"""
    Utilitybelt
    ~~~~~

    :copyright: (c) 2015 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

from collections import defaultdict

""" A recursive dictionary based on the collections lib defaultdict class.
"""
recursive_dict = lambda: defaultdict(recursive_dict)


def recursive_dict_to_dict(rdict):
    """ Convert a recursive dict to a plain ol' dict.
    """
    d = {}
    for (k, v) in rdict.items():
        if isinstance(v, defaultdict):
            d[k] = recursive_dict_to_dict(v)
        else:
            d[k] = v
    return d


def scrub_dict(d):
    """ Recursively inspect a dictionary and remove all empty values, including
        empty strings, lists, and dictionaries.
    """
    if type(d) is dict:
        return dict(
            (k, scrub_dict(v)) for k, v in d.iteritems() if v and scrub_dict(v)
        )
    elif type(d) is list:
        return [
            scrub_dict(v) for v in d if v and scrub_dict(v)
        ]
    else:
        return d


def _to_json_type(obj, classkey=None):
    """ Recursively convert the object instance into a valid JSON type.
    """
    if isinstance(obj, dict):
        data = {}
        for (k, v) in obj.items():
            data[k] = _to_json_type(v, classkey)
        return data
    elif hasattr(obj, "_ast"):
        return _to_json_type(obj._ast())
    elif hasattr(obj, "__iter__"):
        return [_to_json_type(v, classkey) for v in obj]
    elif hasattr(obj, "__dict__"):
        data = dict([
            (key, _to_json_type(value, classkey))
            for key, value in obj.__dict__.iteritems()
            if not callable(value) and not key.startswith('_')
        ])
        if classkey is not None and hasattr(obj, "__class__"):
            data[classkey] = obj.__class__.__name__
        return data
    else:
        return obj


def to_dict(obj):
    """ Convert an instance of an object into a dict.
    """
    d = _to_json_type(obj)
    if isinstance(d, dict):
        return scrub_dict(d)
    else:
        raise ValueError("The value provided must be an object.")



# -*- coding: utf-8 -*-
"""
    Utilitybelt
    ~~~~~

    :copyright: (c) 2015 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

__version__ = '0.2.2'

from .dicts import recursive_dict, scrub_dict, to_dict, recursive_dict_to_dict
from .charsets import int_to_charset, charset_to_int, change_charset
from .charsets import base16_chars, base58_chars, base32_chars, zbase32_chars, \
    base64_chars
from .base16 import hex_to_int, int_to_hex, hex_to_charset, charset_to_hex, \
    hexpad, is_hex, is_int, is_valid_int
from .entropy import dev_urandom_entropy, dev_random_entropy, secure_randint



# -*- coding: utf-8 -*-
"""
    Utilitybelt
    ~~~~~

    :copyright: (c) 2015 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import string
from .charsets import change_charset

B16_CHARS = string.hexdigits[0:16]
B16_REGEX = '^[0-9a-f]*$'


def hex_to_int(s):
    try:
        return int(s, 16)
    except:
        raise ValueError("Value must be in hex format")


def int_to_hex(i):
    try:
        return hex(i).rstrip('L').lstrip('0x')
    except:
        raise ValueError("Value must be in int format")


def is_hex(s):
    # make sure that s is a string
    if not isinstance(s, str):
        return False
    # if there's a leading hex string indicator, strip it
    if s[0:2] == '0x':
        s = s[2:]
    # try to cast the string as an int
    try:
        i = hex_to_int(s)
    except ValueError:
        return False
    else:
        return True


def is_int(i):
    return isinstance(i, (int,long))


def is_valid_int(i):
    if is_int(i):
        return True
    elif isinstance(i, str):
        try:
            int_i = int(i)
        except:
            return False
        else:
            return True
    return False


def hexpad(x):
    return ('0' * (len(x) % 2)) + x


def charset_to_hex(s, original_charset):
    return hexpad(change_charset(s, original_charset, B16_CHARS))


def hex_to_charset(s, destination_charset):
    if not is_hex(s):
        raise ValueError("Value must be in hex format")
    s = s.lower()
    return change_charset(s, B16_CHARS, destination_charset)



# -*- coding: utf-8 -*-
"""
    Utilitybelt
    ~~~~~

    :copyright: (c) 2015 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import os
import binascii
import random


def dev_urandom_entropy(numbytes):
    """ Reads random bytes from the /dev/urandom pool.

        NOTE: /dev/urandom is a non-blocking pseudorandom number generator.
        If the entropy pool runs out, previously released entropy will be used.
        If blocking is unnacceptable, use this over "dev_urandom_entropy".
    """
    return os.urandom(numbytes)


def dev_random_entropy(numbytes, fallback_to_urandom=True):
    """ Reads random bytes from the /dev/random entropy pool.

        NOTE: /dev/random is a blocking pseudorandom number generator.
        If the entropy pool runs out, this function will block until more
        environmental noise is gathered.
        If entropy re-use is unnacceptable use this over "dev_urandom_entropy".

        If "fallback_to_urandom" is set, this function will fallback to
        /dev/urandom on operating systems without /dev/random.
    """
    if os.name == 'nt' and fallback_to_urandom:
        return dev_urandom_entropy(numbytes)
    return open("/dev/random", "rb").read(numbytes)


def secure_randint(min_value, max_value, system_random=None):
    """ Return a random integer N such that a <= N <= b.

        Uses SystemRandom for generating random numbers.
        (which uses os.urandom(), which pulls from /dev/urandom)
    """
    if not system_random:
        system_random = random.SystemRandom()
    return system_random.randint(min_value, max_value)



# -*- coding: utf-8 -*-
"""
    Utilitybelt
    ~~~~~

    :copyright: (c) 2015 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import re
import string


def int_to_charset(val, charset):
    """ Turn a non-negative integer into a string.
    """
    if not val >= 0:
        raise ValueError('"val" must be a non-negative integer.')
    if val == 0:
        return charset[0]
    output = ""
    while val > 0:
        val, digit = divmod(val, len(charset))
        output += charset[digit]
    # reverse the characters in the output and return
    return output[::-1]


def charset_to_int(s, charset):
    """ Turn a string into a non-negative integer.
    """
    output = 0
    for char in s:
        output = output * len(charset) + charset.index(char)
    return output


def change_charset(s, original_charset, target_charset):
    """ Convert a string from one charset to another.
    """
    if not isinstance(s, str):
        raise ValueError('"s" must be a string.')

    intermediate_integer = charset_to_int(s, original_charset)
    output_string = int_to_charset(intermediate_integer, target_charset)
    return output_string


""" Base16 includes numeric digits and the letters a through f. Here,
    we use the lowecase letters.
"""
base16_chars = '0123456789abcdef'

""" The Base58 character set allows for strings that avoid visual ambiguity
    when typed. It consists of all the alphanumeric characters except for
    "0", "O", "I", and "l", which look similar in some fonts.

    https://en.bitcoin.it/wiki/Base58Check_encoding
"""
base58_chars = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"

""" The Base32 character set allows for accurate transcribing by hand.
    It consists of uppercase letters + numerals, excluding "0", "1", + "8",
    which could look similar to "O", "I", and "B" and so are omitted.

    http://en.wikipedia.org/wiki/Base32
"""
base32_chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ234567"

""" The z-base-32 character set is similar to the standard Base32 character
    set, except it uses lowercase letters + numerals and chooses to exclude
    "0", "l", "v", + "2". The set is also permuted so that easier chars
    occur more frequently.

    http://philzimmermann.com/docs/human-oriented-base-32-encoding.txt
"""
zbase32_chars = "ybndrfg8ejkmcpqxot1uwisza345h769"

""" The Base64 character set is a popular encoding for transmitting data
    over media that are designed for textual data. It includes all alphanumeric
    characters plus two bonus characters, usually "+" and "/".

    http://en.wikipedia.org/wiki/Base64
"""
base64_chars = ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
                "0123456789+/")
